@@ -1,4 +1,4 @@
-# Copyright 2019-2021 Cambridge Quantum Computing
+# Copyright 2019-2023 Quantinuum
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,7 +35,7 @@ from .cdr_post import (
     _PolyCDRCorrect,
     cdr_quality_check_task_gen,
 )
-import numpy as np
+import numpy as np  # type: ignore
 import random
 from enum import Enum
 import warnings
@@ -572,7 +572,6 @@ def gen_CDR_MitEx(
     )
     _post_calibrate_task_graph.append(
         cdr_calibration_task_gen(
-            device_backend,
             kwargs.get("model", _PolyCDRCorrect(1)),
         )
     )
@@ -597,6 +596,6 @@ def gen_CDR_MitEx(
         )
     )
     _experiment_taskgraph.append(_post_task_graph)
-    _experiment_taskgraph.append(cdr_correction_task_gen(device_backend))
+    _experiment_taskgraph.append(cdr_correction_task_gen())
 
     return MitEx(device_backend).from_TaskGraph(_experiment_taskgraph)
