@@ -1,4 +1,4 @@
-# Copyright 2019-2021 Cambridge Quantum Computing
+# Copyright 2019-2023 Quantinuum
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@ import itertools
 from functools import lru_cache
 
 from pytket.circuit import Circuit, Qubit, Bit, Node, CircBox  # type: ignore
-import numpy as np
+import numpy as np  # type: ignore
 from collections import OrderedDict, namedtuple
 from typing import Dict, Iterable, List, Tuple, Counter, cast, Optional
 from math import ceil, log2
