@@ -1,4 +1,4 @@
-# Copyright 2019-2021 Cambridge Quantum Computing
+# Copyright 2019-2023 Quantinuum
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -72,7 +72,7 @@ def gen_FullyCorrelated_SPAM_MitRes(
 
     _task_graph_spam_calib.add_wire()
     _task_graph_spam_calib.append(
-        gen_full_tomography_spam_characterisation_task(backend, correlations)
+        gen_full_tomography_spam_characterisation_task(correlations)
     )
 
     _task_graph_spam_correction.parallel(_task_graph_spam_calib)
@@ -81,7 +81,7 @@ def gen_FullyCorrelated_SPAM_MitRes(
     )
     _task_graph_spam_correction.append(
         gen_full_tomography_spam_correction_task(
-            backend, kwargs.get("correction_method", CorrectionMethod.Invert)
+            kwargs.get("correction_method", CorrectionMethod.Invert)
         )
     )
     return MitRes(backend).from_TaskGraph(_task_graph_spam_correction)
